%% Copyright (C) 2001,2002 INRIA Emmanuel Coquery

%% This library is free software; you can redistribute it and/or
%% modify it under the terms of the GNU Lesser General Public
%% License as published by the Free Software Foundation; either
%% version 2.1 of the License, or (at your option) any later version.

%% This library is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.

%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                           %%
%%                  prolog dot file generator                                %%
%%                                                                           %%
%%         Emmanuel Coquery, projet Contraintes, INRIA Rocquencourt          %%
%%                                                                           %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Header: /home/yquem/contraintes/coquery/cvs/contraintes/tclpchr/src/prolog_dot.pl,v 1.1 2002/09/04 14:58:18 coquery Exp $

:- module(prolog_dot,[write_atom_graph/3,write_color_atom_graph/3]).

:- use_module(library(ugraphs)).
:- use_module(library(lists)).

write_atom_graph(Stream,Title,Graph) :-
        format(Stream,'digraph ~a {~n',Title),
        edges(Graph, Edges),
        write_atom_edges(Stream,Edges),
        format(Stream,'}~n',[]).

write_atom_edges(_,[]).
write_atom_edges(Stream,[U-V|Rest]) :-
        format(Stream, '  "~a" -> "~a";~n', [U,V]),
        write_atom_edges(Stream,Rest).

write_color_atom_graph(Stream,Title,Graph) :-
        format(Stream,'digraph ~a {~n',Title),
        vertices(Graph,Vertices),
        write_color_vertices(Stream,Vertices),
        edges(Graph, Edges),
        write_color_atom_edges(Stream,Edges),
        retractall(vertice_color(_,_)),
        format(Stream,'}~n',[]).

:- dynamic vertice_color/2.

write_color_vertices(Stream,Vertices) :-
        first_color(C),
        write_color_vertices(Stream,Vertices,C).
write_color_vertices(_,[],_).
write_color_vertices(Stream,[V|Rest],C) :-
        assert(vertice_color(V,C)),
        format(Stream,'  "~a" [color = "~a"];~n',[V,C]),
        next_color(C,NC),
        write_color_vertices(Stream,Rest,NC).

write_color_atom_edges(_,[]).
write_color_atom_edges(Stream,[U-V|Rest]) :-
        vertice_color(U,Color),
        format(Stream, '  "~a" -> "~a" [color = "~a"];~n', [U,V,Color]),
        write_color_atom_edges(Stream,Rest).

next_color(C1,C2) :-
        colors(Colors),
        (
          nextto(C1,C2,Colors);
          Colors = [ C2 | _ ],last(Colors,C1)
        ).

first_color(C) :-
        colors([C|_]).

%colors([black,gray,red,pink,beige,khaki,orange,
%        yellow,green,seagreen,aquamarine,blue,violet]).
colors([black, brown,
        darkgoldenrod, chartreuse, aquamarine,
        blueviolet, crimson, burlywood, orange, gold,
        darkgreen, cyan, blue, darkorchid,darksalmon,chocolate,
        goldenrod,darkolivegreen,darkturquoise,
        cadetblue,darkviolet,deeppink,darkkhaki,greenyellow,
        darkseagreen,darkslateblue,magenta]).

simple_graph(G) :-
        vertices_edges_to_ugraph([a,b,c,d],[a-b,b-c,c-d,b-d],G).

test :-
        simple_graph(G), open('test.dot',write,S), write_atom_graph(S,'G',G),
        close(S).
